// AppendProjection.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "AppendProjection.h"

#include "geotiff.h"
#include "xtiffio.h"
#include "geo_normalize.h"
#include "geo_simpletags.h"
#include "geovalues.h"
#include "tiffio.h"
#include "cpl_serv.h"
#include <stdio.h>



#define TIFFOpen XTIFFOpen
#define TIFFClose XTIFFClose

#if defined(VMS)
#define unlink delete
#endif

#define	streq(a,b)	(strcmp(a,b) == 0)
#define	strneq(a,b,n)	(strncmp(a,b,n) == 0)

#define	TRUE	1
#define	FALSE	0



#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

using namespace std;

static void GTIFPrintCorners( GTIF *gtif, GTIFDefn *defn, FILE * fp_out,
							 int xsize, int ysize, int inv_flag, int dec_flag );

static int GTIFReportACorner( GTIF *gtif, GTIFDefn *defn, FILE * fp_out,
							 const char * corner_name,
							 double x, double y, int inv_flag, int dec_flag );

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	int	  norm_print_flag = 1, proj4_print_flag = 0;
    int		tfw_flag = 0, inv_flag = 0, dec_flag = 0;
    int         st_test_flag = 0;
	


	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		nRetCode = 1;
	}
	else
	{

		char *geotiffname = NULL;
		
		geotiffname = argv[1];

		TIFF 	*tif=(TIFF*)0;  /* TIFF-level descriptor */
		GTIF	*gtif=(GTIF*)0; /* GeoKey-level descriptor */

		
		tif=XTIFFOpen(geotiffname,"a");

		gtif = GTIFNew(tif);
        if (!gtif)
        {
            fprintf(stderr,"failed in GTIFNew\n");
			return 0;
           // goto failure;
        }

		GTIFDefn	defn;
		if( GTIFGetDefn( gtif, &defn ) )
        {

			GTIFPrint(gtif,0,0);

			printf( "\n" );
            GTIFPrintDefn( &defn, stdout );


		//	const char *prjinfo = "+proj=Transverse_Mercator+Spheroid=GRS 1980+datum=D_NZGD_2000+Scale_Factor=0.999600+Central_Meridian=173.0+Latitude_Of_Origin=0.0+False_Easting=1600000+False_Northing=10000000";

		//	const char *prjinfo = "+proj=tmerc+ellps=GRS 1980+datum=NZGD2000+k=0.999600+lon_0=173.0+lat_0=0.0+x_0=1600000+y_0=10000000+units=m";

			const char *prjinfo = "+proj=tmerc+lat_0=0.0+lon_0=173.0+k=0.999600+x_0=1600000.0+y_0=10000000.0";


			//char **papszNV = OSRProj4Tokenize(prjinfo);


			if( !GTIFSetFromProj4(gtif,prjinfo) )
			{
				fprintf(stderr,"Failure in GTIFSetFromProj4\n");
				exit (-1);
			}


			//GTIFSetFromProj4(gtif,prjinfo);

			
//             printf( "\n" );
//             printf( "PROJ.4 Definition: %s\n", GTIFGetProj4Defn(&defn));
// 
// 			 int		xsize, ysize;
// 
// 				TIFFGetField( tif, TIFFTAG_IMAGEWIDTH, &xsize );
// 				TIFFGetField( tif, TIFFTAG_IMAGELENGTH, &ysize );
// 				GTIFPrintCorners( gtif, &defn, stdout, xsize, ysize, inv_flag, dec_flag );



		}
		GTIFWriteKeys(gtif);
		GTIFFree(gtif);


	
	}

	return nRetCode;
	
//failure:
   // fprintf(stderr,"failure in listgeo\n");

}

static void GTIFPrintCorners( GTIF *gtif, GTIFDefn *defn, FILE * fp_out,
							 int xsize, int ysize, int inv_flag, int dec_flag )
							 
{
    printf( "\nCorner Coordinates:\n" );
    if( !GTIFReportACorner( gtif, defn, fp_out,
		"Upper Left", 0.0, 0.0, inv_flag, dec_flag ) )
    {
        printf( " ... unable to transform points between pixel/line and PCS space\n" );
        return;
    }
	
    GTIFReportACorner( gtif, defn, fp_out, "Lower Left", 0.0, ysize, 
		inv_flag, dec_flag );
    GTIFReportACorner( gtif, defn, fp_out, "Upper Right", xsize, 0.0,
		inv_flag, dec_flag );
    GTIFReportACorner( gtif, defn, fp_out, "Lower Right", xsize, ysize,
		inv_flag, dec_flag );
    GTIFReportACorner( gtif, defn, fp_out, "Center", xsize/2.0, ysize/2.0,
		inv_flag, dec_flag );
}

static int GTIFReportACorner( GTIF *gtif, GTIFDefn *defn, FILE * fp_out,
							 const char * corner_name,
							 double x, double y, int inv_flag, int dec_flag )
							 
{
    double	x_saved, y_saved;
	
    /* Try to transform the coordinate into PCS space */
    if( !GTIFImageToPCS( gtif, &x, &y ) )
        return FALSE;
    
    x_saved = x;
    y_saved = y;
	
    fprintf( fp_out, "%-13s ", corner_name );
	
    if( defn->Model == ModelTypeGeographic )
    {
		if (dec_flag) 
		{
			fprintf( fp_out, "(%.7f,", x );
			fprintf( fp_out, "%.7f)\n", y );
		} 
		else 
		{
			fprintf( fp_out, "(%s,", GTIFDecToDMS( x, "Long", 2 ) );
			fprintf( fp_out, "%s)\n", GTIFDecToDMS( y, "Lat", 2 ) );
		}
    }
    else
    {
        fprintf( fp_out, "(%12.3f,%12.3f)", x, y );
		
        if( GTIFProj4ToLatLong( defn, 1, &x, &y ) )
        {
			if (dec_flag) 
			{
                fprintf( fp_out, "  (%.7f,", x );
                fprintf( fp_out, "%.7f)", y );
			} 
			else 
			{
				fprintf( fp_out, "  (%s,", GTIFDecToDMS( x, "Long", 2 ) );
				fprintf( fp_out, "%s)", GTIFDecToDMS( y, "Lat", 2 ) );
			}
        }
		
        fprintf( fp_out, "\n" );
    }
	
    if( inv_flag && GTIFPCSToImage( gtif, &x_saved, &y_saved ) )
    {
        fprintf( fp_out, "      inverse (%11.3f,%11.3f)\n", x_saved, y_saved );
    }
    
    return TRUE;
}
